/* serial.h								*/
/*									*/
/* Copyright (C) 1994 by Robert Armstrong				*/
/*									*/
/* This program is free software; you can redistribute it and/or modify	*/
/* it under the terms of the GNU General Public License as published by	*/
/* the Free Software Foundation; either version 2 of the License, or	*/
/* (at your option) any later version.					*/
/*									*/
/* This program is distributed in the hope that it will be useful, but	*/
/* WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANT-	*/
/* ABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General	*/
/* Public License for more details.					*/
/*									*/
/* You should have received a copy of the GNU General Public License	*/
/* along with this program; if not, visit the website of the Free	*/
/* Software Foundation, Inc., www.gnu.org.				*/


/*   These symbols define the registers and bits of the UART chips	*/
/* commonly found in the IBM PC.  The 8250 is the "traditional" chip	*/
/* used in the original PC.  It is now rarely used in new hardware, but	*/
/* remains the lowest common denominator.  The 8250 is expecially	*/
/* annoying to programmers because it contains a number of hardware 	*/
/* bugs which we must program around!  The 16450 is simply a faster	*/
/* version of the 8250, and as far as the programmer knows they	are the	*/
/* same (although the 16450 fixes most of the 8250's bugs!). The 16550	*/
/* is a 16450 with a FIFO buffer added. Definitions which are unique to	*/
/* the 16550 are marked as such.					*/

/*   UART register addresses.  These values are offsets which must be	*/
/* added to the appropriate COM port base address to obtain the actual	*/
/* address.  Note that many times a different register is accessed	*/
/* depending on whether a read or a write operation is done (e.g. RBR	*/
/* and THR, IIR and FCR).  Also, the DLL and DLH registers overlap the	*/
/* first two registers - the DLx registers can only be accessed when	*/
/* the DLAB bit is set in the line control register.			*/
#define RBR		0	/* receiver buffer register		*/
#define TBR		0	/* transmitter "       "		*/
#define IER		1	/* interrupt enable register		*/
#define IIR		2	/* interrupt identification register	*/
#define FCR		2	/* FIFO control register [16550 only]	*/
#define LCR		3	/* line control register		*/
#define MCR		4	/* modem control register		*/
#define LSR		5	/* line status register			*/
#define MSR		6	/* modem status register		*/
#define DLL		0	/* baud rate divisor latch (low byte)	*/
#define DLH		1	/*   "    "    "       "   (high  " )	*/

/* Bits defined in the IER (interrupt enable register) ...		*/
#define IER_RBF		0x01	/* interrupt on receiver buffer full	*/
#define IER_TBE		0x02	/*     "     "  transmitter "   empty	*/
#define IER_SER		0x04	/*     "     "  error (parity framing)	*/
#define IER_MDM		0x08	/*     "     "  modem status change	*/

/* Bits defined in the IIR (interrupt identification register) ...	*/
#define IIR_IRQ		0x01	/* CLEARED if any interrupt pending	*/
#define IIR_TYP		0x06	/* interrupt type mask			*/
#define    IIR_MDM	0x00	/*   modem status change		*/
#define    IIR_TBE	0x02	/*   transmitter buffer empty		*/
#define    IIR_RBF	0x04	/*   receiver buffer full		*/
#define    IIR_SER	0x06	/*   error (parity, framing, etc)	*/
#define IIR_FEN1	0x40	/* FIFO enabled	[16550 only]		*/
#define IIR_FEN2	0x80	/*   "     "	[16550 only]		*/

/* Bits defined in the FCR (FIFO control register) ...			*/
/* NOTE: this entire register is 16550 only!!!				*/
#define FCR_ENB		0x01	/* FIFO enable				*/
#define FCR_RFR		0x02	/* receiver FIFO reset			*/
#define FCR_TFR		0x04	/* transmitter "   "			*/
#define FCR_DMA		0x08	/* DMA mode select			*/
#define FCR_FTH		0xC0	/* receiver FIFO threshold mask		*/
#define    FCR_F1	0x00	/*   interrupt when FIFO is non-empty	*/
#define    FCR_F4	0x40	/*       "       "  4 bytes in FIFO	*/
#define    FCR_F8	0x80	/*       "       "  8   "    "   "	*/
#define    FCR_F14	0xC0	/*       "       " 14   "    "   "	*/

/* Bits defined in the LCR (line control register) ...			*/
#define LCR_BITS	0x03	/* data bit count mask			*/
#define    LCR_5BITS	0x00	/*   5 data bits			*/
#define    LCR_6BITS	0x01	/*   6  "    "				*/
#define    LCR_7BITS	0x02	/*   7  "    "				*/
#define    LCR_8BITS	0x03	/*   8  "    "				*/
#define LCR_2STOP	0x04	/* use two stop bits (otherwise use 1	*/
#define LCR_PARITY	0x38	/* parity mask				*/
#define    LCR_PARNON	0x00	/*   no parity				*/
#define    LCR_PARODD	0x08	/*   odd   "				*/
#define    LCR_PAREVE	0x18	/*   even  "				*/
#define    LCR_PARMAR	0x28	/*   mark  "				*/
#define    LCR_PARSPA	0x38	/*   space "				*/
#define LCR_BREAK	0x40	/* force a break state when set		*/
#define LCR_DLAB	0x80	/* divisor latch access enable bit	*/

/* Bits defined in the MCR (modem control register) ...			*/
#define MCR_DTR		0x01	/* data terminal ready state		*/
#define MCR_RTS		0x02	/* request to send state		*/
#define MCR_GPO1	0x04	/* general purpose output #1 state	*/
#define MCR_GPO2	0x08	/*    "       "       "   #2   "	*/
#define MCR_LLT		0x10	/* enable local loopback testing	*/

/* Bits defined in the LSR (line status register) ...			*/
#define LSR_RBF		0x01	/* receiver buffer full			*/
#define LSR_ROV		0x02	/* receiver buffer overrun		*/
#define LSR_PER		0x04	/* received data parity error detected	*/
#define LSR_FER		0x08	/*    "      "   framing  "      "	*/
#define LSR_BRK		0x10	/* received signal break detected	*/
#define LSR_TBE		0x20	/* transmitter buffer empty		*/
#define LSR_TXE		0x40	/* transmitter shift register empty	*/
#define LSR_EFB		0x80	/* [16550] error in FIFO buffer		*/

/* Bits defined in the MSR (modem status register) ...			*/
#define MSR_DCTS	0x01	/* change in clear to send detected	*/
#define MSR_DDSR	0x02	/*    "    " data set ready   "		*/
#define MSR_DRI		0x04	/*    "    " ring indicator   "		*/
#define MSR_DDCD	0x08	/*    "    " data carrier     "		*/
#define MSR_CTS		0x10	/* current state of clear to send	*/
#define MSR_DSR		0x20	/*    "      "   "  data set ready	*/
#define MSR_RI		0x40	/*    "      "   "  ring indicator	*/
#define MSR_DCD		0x80	/*    "      "   "  data carrier detect	*/


/* Serial port (COMn) related definitions ...				*/
#define COM1VEC		0x0C	/* interrupt vector used by COM 1 & 3	*/
#define COM2VEC		0x0B	/*     "       "      "  "   "  2 & 4	*/
#define MAXBAUD	(long)	115200	/* constant for baud rate divisor	*/


/* 8259 PIC definitions (programmable interrupt controller) ...		*/
/*   NOTE: On the PC/XT this is the _only_ PIC.  On the PC/AT, there 	*/
/* are two PICs, and these definitions refer to the slave 8259.  The	*/
/* port addresses and bits are the same in either case...		*/
#define PIC_ICR		0x20	/* interrupt control register port	*/
#define    ICR_EOI	0x20	/*   end of interrupt control code	*/
#define PIC_IMR		0x21	/* interrupt mask register port		*/
#define    IMR_COM2	0x08	/*   mask for COM2 and COM4 interrupts	*/
#define    IMR_COM1	0x10	/*    "    "  COM1 and COM3     "	*/


/* COMPut - add one byte to the transmitter buffer */
BOOLEAN COMPut (BYTE);
/* COMPutW - like Put(), but wait if the buffer is full */
void COMPutW (BYTE);
/* COMGet - get one byte from the receive buffer */
BOOLEAN COMGet (BYTE far *);
/* COMGetW - like Get(), but wait with a timeout */
BOOLEAN COMGetW (BYTE far *, WORD);
/* COMPeekW - return next byte from buffer without removing it */
BOOLEAN COMPeekW (BYTE far *, WORD);
/* COMFormat - set data format (parity, number of bits, etc) */
void COMFormat (char *);
/* COMBaud - set the baud rate */
void COMBaud (WORD);
/* COMBreak - force a break condition on the TxD signal */
void COMBreak (void);
/* COMOpen - open the port and seize the interrupt vector */
BOOLEAN COMOpen (int);
/* COMClose - close the port and restore the interrupt */
void COMClose (void);
