PAGE 60, 132
     	TITLE	UARTISR - Serial Port Interrupt Service
	SUBTTL	Bob Armstrong [17-Aug-94]

; uartisr.asm
;
; Copyright (C) 1994 by Robert Armstrong
;
; This program is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; This program is distributed in the hope that it will be useful, but
; WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANT-
; ABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
; Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with this program; if not, visit the website of the Free
; Software Foundation, Inc., www.gnu.org.


;   This module contains the interrupt service routines needed for
; the PC COM ports.  It works in conjunction with the SERIAL.C file
; to create a complete COM port driver.  You can actually write a
; working version of this completely in Turbo-C, but it has two
; problems: (1) it's a little slow because the compiler generated
; code isn't that efficient, and (2) the way in which Turbo-C loads
; the data segment register doesn't translate into .COM file format.
; Hence this version...
;
;					Bob Armstrong [17-Aug-94]

	.8086

RXBUFSIZ	EQU	128	; receiver circular buffer size, in bytes
TXBUFSIZ	EQU	32	; transmitter  "       "     "   "   "


;   This is a limited subset of the 8250/16x50 UART and 8259 PIC
; register definitions.  A more complete set is contained in
; SERIAL.C - these are just what are needed to assemble this file.
IER		EQU	1	; interrupt enable register
   IER_TBE	EQU	002H	;   enable transmitter buffer empty interrupts
IIR		EQU	2	; interrupt identification register
   IIR_IRQ	EQU	001H	;   CLEARED if any interrupt pending
   IIR_TYP	EQU	006H	;   interrupt type mask
LSR		EQU	5	; line status register
   LSR_RBF	EQU	001H	;   receiver buffer full
   LSR_TBE	EQU	020H	;   transmitter buffer empty
MSR		EQU	6	; modem status register
PIC_ICR		EQU	020H	; interrupt control register port
   ICR_EOI	EQU	020H	;   end of interrupt control code


; Data storage for this module...
DGROUP	GROUP	_DATA, _BSS
_BSS	SEGMENT PUBLIC 'BSS'

;   These variables define the circular buffers used for transmitting and
; receiving.  They are shared between the interrupt service routines in
; this module and the C routines in the SERIAL.C module.
	PUBLIC	_rxgetp, _rxputp, _rxbuf
	PUBLIC	_txgetp, _txputp, _txbuf
_rxgetp	DW	?		; next byte to fill in receive buffer
_rxputp	DW	?		;  "    "   "  empty "   "       "
_txgetp	DW	?		; next byte to fill in transmit buffer
_txputp	DW	?		;  "    "   "  empty "     "      "
_txbuf	DB	TXBUFSIZ DUP (?); circular buffer for transmitting
_rxbuf	DB	RXBUFSIZ DUP (?);  "   "      "    "  receiving

; Local storage for this module only...
PORT	DW	?		; I/O port address for our UART (e.g. 3F8, 2F8)
_BSS	ENDS

_DATA	SEGMENT PUBLIC 'DATA'

; The C code uses these values to know the actual sizes of the buffers...
	PUBLIC	_rxbufsiz, _txbufsiz
_rxbufsiz	DW	RXBUFSIZ; size of the receiver buffer
_txbufsiz	DW	TXBUFSIZ;  "   "   "  transmitter "

_DATA	ENDS


_TEXT	SEGMENT PUBLIC 'CODE'
	ASSUME	CS:_TEXT,DS:DGROUP


;   This word (which must be in the code segment!) is loaded by setup_isr()
; with the segment address of DGROUP.  The interrupt service routine uses
; this to load the DS register so we can access the shared variables.
DGROUP@	DW	0


; void near setup_isr (WORD vector, WORD port);
;
;   This routine will install the address of the interrupt service
; routine in the appropriate hardware vector.  It also saves copies
; of the current data segment address, and the UART base I/O address,
; for the interrupt service routines to use.  Naturally it must be
; called before interrupts can be enabled!!

	PUBLIC	_setup_isr
_setup_isr	PROC	NEAR
	PUSH	BP		; standard C routine entry prolog
	MOV	BP,SP		; ...
	MOV	CS:DGROUP@,DS	; save the address of DGROUP for the ISR
	MOV	AX,[BP+6]	; load the UART I/O port address
	MOV	PORT,AX		; and also remember that for the ISR
	MOV	AX,[BP+4]	; now get the hardware vector address
	MOV	AH,25H		; DOS function 25H - set interrupt vector
	PUSH	DS		; save the current DS
	MOV	BX,CS		; and load DS:DX with the ISR address
	MOV	DS,BX		; ...
	MOV	DX,OFFSET _TEXT:_uart_isr
	INT	21H		; ask DOS to set the vector for us
	POP	DS		; restore DS and return
	POP	BP		; ...
	RET			; ...
_setup_isr	ENDP


; void near xmit_isr (void);
;
;   This routine will service transmitter buffer empty interrupts.
; If there is more data in the buffer, then we'll send it.  If the
; buffer is now empty, then we'll turn off the transmitter interrupt
; enable.  It gets turned back on when data is added to the buffer.
;
;   Some UARTs (e.g. the 16550A) will have hardware FIFOs that the
; C code will have enabled.  In this case the UART may be able to
; accept multiple characters per interrupt.  We just keep looping
; and feeding it data as long as the TBE bit remains set.  This
; algorithm also works fine on non-FIFO uarts.
;
;   Note that this routine has no standard prolog or epilog because
; it has no parameters nor local variables.  It may use the AX, BX
; and DX registers only!

	PUBLIC	_xmit_isr
_xmit_isr	PROC	NEAR

XMIT1:	MOV	DX,PORT		; read the UART line status register
	ADD	DX,LSR		; ...
 	IN	AL,DX		; ...
	TEST	AL,LSR_TBE	; is the transmitter buffer empty ?
	JE	XMIT99		; no - time to leave now!
	MOV	AX,_txgetp	; are there any bytes waiting in the buffer ?
	CMP	AX,_txputp	; ???
	JNZ	XMIT2		; yes - go send the next one

;   There is no more data to transmit.  Be sure to clear the transmitter
; interrupt enable bit before returning - otherwise we'll be hung up here!
	MOV	DX,PORT		; address the UART interrupt enable register
	ADD	DX,IER		; ...
	IN	AL,DX		; read the current contents
	AND	AL,NOT IER_TBE	; clear the transmitter enable flag
	OUT	DX,AL		; ...
	JMP	XMIT99

; There's data in the buffer and the transmitter's ready...
XMIT2:	INC	_txgetp		; advance the buffer pointer
	MOV	AX,_txgetp	; have we reached the end of the buffer ?
	CMP	AX,_txbufsiz	; ??
	JB	XMIT21		; ??
	MOV	_txgetp,0	; yes - wrap around
XMIT21:	MOV	BX,_txgetp	; fetch the next byte from the buffer
	MOV	AL,_txbuf[BX]	; ...
	MOV	DX,PORT		; the UART data port is always at offset 0!
 	OUT	DX,AL		; send it another byte
	JMP	XMIT1		; and keep filling the FIFO if we can

XMIT99:	RET	

_xmit_isr	ENDP


; void near recv_isr (void)
;
;   This routine will service receiver data available interrupts.
; It simply reads the byte from the UART and stores it in the
; circular buffer, assuming space is available.  If the buffer is
; full then we simply throw away the data (I think it's called
; "load shedding" :-)  The receiver may have a hardware FIFO, much
; like the transmitter, and we handle it in the same way.
;
;   Note that this routine has no standard prolog or epilog because
; it has no parameters nor local variables.  It may use the AX, BX
; and DX registers only!

	PUBLIC	_recv_isr
_recv_isr	PROC	NEAR

RECV1:	MOV	DX,PORT		; read the UART line status register
	ADD	DX,LSR		; ...
 	IN	AL,DX		; ...
	TEST	AL,LSR_RBF	; anything in the receiver buffer ?
	JNZ	RECV2		; yes - go get it

;   There's a hardware bug in all the UART implementations - if
; both a receiver and a transmitter interrupt occur at the same
; instant, then the transmitter interrupt can be lost.  The
; easiest solution is just to test the transmitter empty bit here
; and call xmit_isr() if it's set.
	TEST	AL,LSR_TBE	; is the transmitter ready for data ?
	JE	RECV99		; just return if not
	MOV	AX,_txgetp	; yes - is there anything to send ?
	CMP	AX,_txputp	; ??
	JE	RECV99		; just return if not
	CALL	_xmit_isr	; yes - start the transmitter now
	JMP	RECV99		; and then return

;   Read the character and put it in the buffer (if we can).  It is
; critical that we read the buffer even if the buffer is full - if
; we don't then the interrupt request is not cleared!
RECV2:	MOV	DX,PORT		; the data register is always at offset 0!
 	IN	AL,DX		; read the byte from the UART
	MOV	BX,_rxputp	; get the current buffer pointer
	INC	BX		; and pre-increment it
	CMP	BX,_rxbufsiz	; do we need to wrap around ?
	JB	RECV20		; jump if not
	XOR	BX,BX		; yes - back to zero
RECV20:	CMP	BX,_rxgetp	; is the buffer full ?
	JE	RECV1		; just skip storing if it is
	MOV	_rxputp,BX	; no - update the buffer pointer
	MOV	_rxbuf[BX],AL	; and store the data byte
	JMP	RECV1		; see if there's more in the FIFO

RECV99:	RET	

_recv_isr	ENDP



; void near msr_isr (void)
; void near lsr_usr (void)
;
;   These two interrupts are not enabled and should never occur
; so these routines are really "just in case."  The problem is that
; if one of these interrupts should ever happen then we need to read
; the corresponding UART register to clear the interrupt request.
; Otherwise we'll be hung!
;
;   These routines have no prolog or epilog and use the AX and DX
; registers.

_msr_isr	PROC	NEAR
	MOV	DX,PORT		; read the UART modem status register
	ADD	DX,MSR		; ...
 	IN	AL,DX		; and just discard the results!
	RET			; ...
_msr_isr	ENDP

_lsr_isr	PROC	NEAR
	MOV	DX,PORT		; read the UART line status register
	ADD	DX,LSR		; ...
 	IN	AL,DX		; and we don't care what it says!
	RET			; ...
_lsr_isr	ENDP


; void interrupt uart_isr (void)
;
;   This routine handles all UART interrupt functions.  The C code will
; change the hardware interrupt vector for the COM port to point here
; when it opens the port.  We need to save the registers, determine
; the cause of the interrupt, and dispatch to the appropriate routine.
; We also need to setup the DS register so that the code can access the
; buffer data and pointer variables.
;
;   Note that it doesn't really matter whether this routine is declared
; as NEAR or FAR, since it's only called by an interrupt and IRET always
; does the same thing.
;
;   Currently we only use and save the AX, BX, DX and DS registers.
; Be careful of this if you add any code!!!

	PUBLIC	_uart_isr
_uart_isr	PROC	FAR

	PUSH	AX		; save the working registers we need
	PUSH	BX		; ...
	PUSH	DX		; ...
	PUSH	DS		; ...
	MOV	AX,CS:DGROUP@	; setup our data segment
	MOV	DS,AX		; ...
	MOV	AL,ICR_EOI	; tell the 8259 PIC that the interrupt is
 	OUT	PIC_ICR,AL	;  ... serviced, to avoid hanging here!

; Extract the cause of the interrupt and call the right routine...
UART01:	MOV	DX,PORT		; read the UART interrupt identification
	ADD	DX,IIR		;  ... register
 	IN	AL,DX		;  ...
	TEST	AL,IIR_IRQ	; did the UART really ask for an interrupt ?
	JNZ	DISMIS		; no - this should never happen!
	AND	AX,IIR_TYP	; isolate just the reason bits
	MOV	BX,AX		; then put it in the index register
	CALL WORD PTR CS:ISRTBL[BX]
	JMP	UART01		; see if there is still more to do

; The interrupt service is over - restore the registers and return...
DISMIS:	POP	DS		; ...
	POP	DX		; ...
	POP	BX		; ...
	POP	AX		; ...
	IRET			; ...

_uart_isr	ENDP


; This table gives the address of the ISR for each UART interrupt condition.
ISRTBL	LABEL	WORD
	DW	_msr_isr	; modem status interrupt
	DW	_xmit_isr	; transmitter ready interrupt
	DW	_recv_isr	; data received interrupt
	DW	_lsr_isr	; line status (error) interrupt


_TEXT	ENDS
	END
